/***********************************************/
/* Software Created By CKJ Vincent Cedric      */
/* This code is the property of Vincent Cedric */
/* You can't copy or use this code without     */
/* according of the author (GPL License).	   */
/***********************************************/
/******************************/
/* classe ColumnView speciale */
/******************************/
#include "CColumnView.h"
#include "CColumnItem.h"

// pour les couleurs
#include "CGlobalDefs.h"

#include <interface/ScrollView.h>
#include <interface/ListView.h>
#include <app/Cursor.h>
#include <interface/Window.h>
#include <app/AppDefs.h>
#include <math.h>

/**** constructeur ****/
CColumnView::CColumnView(const char *name,BCursor *resizeCursor,bool lockColumn = false,bool lastColumnComplete = false)
: BView(BRect(0,0,0,0),name,B_FOLLOW_LEFT_RIGHT,B_WILL_DRAW | B_FRAME_EVENTS)
{	
	// initialisation
	_lastColumnComplete = lastColumnComplete;
	_lockColumn = lockColumn;
	_columnIsInResize = false;
	_resizeCursor = resizeCursor;
	_attachFirstTime = true;
}

/**** destructeur ****/
CColumnView::~CColumnView()
{
	int32		nbItem;
	
	// liberer la memoire
	nbItem = _itemListRect.CountItems();
	for(int32 index = 0;index<nbItem;index++)	
		delete (BRect *)(_itemListRect.RemoveItem((int32)0));

	// le curseur
	if(_resizeCursor!=NULL)
		delete _resizeCursor;
}

/**** attachement a la vue parent ****/
void CColumnView::AttachedToWindow()
{
	// fonction parente
	BView::AttachedToWindow();
	
	// ca doit etre le premier appel
	if(!_attachFirstTime)
		return;

	BScrollView		*target = NULL;
	BRect			frame;
	BRect			bounds;
	int32			nbItems;

	// trouver la scrollView
	_attachFirstTime = false;
	target = dynamic_cast<BScrollView *>(Parent());
	if(target==NULL)
		return;
		
	// redimentionner en fonction de la cible
	frame = target->Bounds();
	ResizeTo(frame.Width() - B_V_SCROLL_BAR_WIDTH - 4,B_V_SCROLL_BAR_WIDTH + 2);
	MoveTo(2,2);
	bounds = Bounds();
	
	// deplacer la vue contenu dans la scroll view
	_listView = dynamic_cast<BListView *>(target->ChildAt(0));	
	if(_listView==NULL)
		return;

	// faire de la place pour notre vue
	_listView->ResizeBy(0,-(bounds.Height()+1));
	_listView->MoveBy(0,(bounds.Height()+1));
	
	// remettre les rectangle a la bonne taille
	nbItems = _itemList.CountItems();
	for(int32 index=0;index<nbItems;index++)
		((BRect *)(_itemListRect.ItemAt(index)))->bottom = bounds.Height() - 1;

}

/**** dessin ****/
void CColumnView::Draw(BRect updateRect)
{
	BRect		bounds = Bounds();
	BRect		itemRect;
	CColumnItem	*current = NULL;
	int32		nbItems;
	float		positionLastRect = SIZE_START_COLOMN;

	// les limites foncées	
	SetHighColor(DARK_4_COLOR);
	StrokeLine(BPoint(0,bounds.top),BPoint(bounds.right,bounds.top));
	StrokeLine(BPoint(0,bounds.bottom),BPoint(bounds.right,bounds.bottom));

	bounds.InsetBy(0,1);
	// cadre gauche
	SetHighColor(WHITE_COLOR);
	StrokeLine(BPoint(0,2),BPoint(10,2));
	SetHighColor(DARK_2_COLOR);
	StrokeLine(BPoint(0,bounds.bottom),BPoint(10,bounds.bottom));
	StrokeLine(BPoint(10,bounds.top),BPoint(10,bounds.bottom));

	SetHighColor(VIEW_COLOR);
	FillRect(BRect(0,2,9,bounds.bottom-1));

	// dessiner les items
	nbItems = _itemList.CountItems();
	for(int32 index=0;index<nbItems;index++)
	{
		current = (CColumnItem *)(_itemList.ItemAt(index));
		itemRect = *((BRect *)(_itemListRect.ItemAt(index)));
		if(current!=NULL)
		{
			SetHighColor(DARK_4_COLOR);
			StrokeLine(BPoint(itemRect.left,1),BPoint(itemRect.left,bounds.bottom));
			itemRect.left++;
			if(_lastColumnComplete && index==(nbItems-1))
				itemRect.right = bounds.right + 1;
			current->DrawItem(this,itemRect);
		}
		positionLastRect += itemRect.Width() + 2;
	}	

	// cadre droit
	SetLowColor(WHITE_COLOR);
	if(bounds.right>=positionLastRect)
	{
		SetHighColor(DARK_4_COLOR);
		StrokeLine(BPoint(positionLastRect,1),BPoint(positionLastRect,bounds.bottom));
		
		positionLastRect++;
		SetHighColor(WHITE_COLOR);
		StrokeLine(BPoint(positionLastRect,1),BPoint(bounds.right,1));
		StrokeLine(BPoint(positionLastRect,1),BPoint(positionLastRect,bounds.bottom));
		SetHighColor(DARK_2_COLOR);
		StrokeLine(BPoint(positionLastRect,bounds.bottom),BPoint(bounds.right,bounds.bottom));

		positionLastRect++;
		SetHighColor(VIEW_COLOR);
		FillRect(BRect(positionLastRect,2,bounds.right,bounds.bottom-1));
	}
}

/**** clique souris ****/
void CColumnView::MouseDown(BPoint point)
{
	// on doit ne pas avoir bloqué le redimentionnement
	if(_lockColumn)
		return;
			
	//_columnIsInResize
	if(!Window()->IsActive())
		return;

	// on doit avoir une colonne de selectionné
	int32	columnIndex;
	
	columnIndex = InColumnResizeArea(point);
	if(columnIndex<0)
		return;

	BPoint	where;
	uint32	buttons;
	
	while(true)
	{
		// recuperer l'emplacement de la souris
		// et l'etat des boutons
		GetMouse(&where, &buttons);
	
		if(buttons==0)
			break;	
		
		// redimentionner
		if(ResizeColumnTo(where,columnIndex)!=B_OK)
			break;

		// relacher un peu le thread
		snooze(15000);
	}

	// rafraichir une derniere fois la vue
	_listView->Invalidate();
}

/**** deplacement de la souris ****/
void CColumnView::MouseMoved(BPoint point,uint32 transit,const BMessage *message)
{
	// on doit ne pas avoir bloqué le redimentionnement
	if(_lockColumn)
		return;
		
	// on doit avoir un curseur de defini
	if(_resizeCursor==NULL)
		return;
		
	switch(transit)
	{
	// si on quitte la vue on resize pas !
	case B_EXITED_VIEW:
		SetViewCursor(B_CURSOR_SYSTEM_DEFAULT);
		break;
	default:
		// on est peux etre dans le coté droit d'une colonne
		if(InColumnResizeArea(point)>-1 && Window()->IsActive())
			SetViewCursor(_resizeCursor);
		else
			SetViewCursor(B_CURSOR_SYSTEM_DEFAULT);
		break;			
	}
}

/**** redimentionner la colonne a partir d'un index de colonne ****/
status_t CColumnView::ResizeColumnTo(BPoint &where,int32 &startIndex)
{
	BRect		*resizeRect = NULL;
	
	// verifier que le rectangle est dans la liste
	resizeRect = (BRect *)(_itemListRect.ItemAt(startIndex));
	if(resizeRect==NULL)
		return B_ERROR;

	int32	offset;
	
	offset = (int32)(where.x - resizeRect->right);
	if(offset!=0)
	{
		BRect			*columnRect = NULL;
		BRect			bounds = Bounds();
		CColumnItem		*column = NULL;
		int32			labelSize;
		
		// verifier que l'on verra toujours le texte
		column = (CColumnItem *)(_itemList.ItemAt(startIndex));
		labelSize = (int32)(StringWidth(column->Label()) + SIZE_LABEL_MORE);
		
		// verifier que la taille n'est pas inferieur au texte
		if((resizeRect->Width() + offset)<=labelSize)
			return B_OK;

		// redimentionner le rectangle
		resizeRect->right += offset;
		
		// deplacer les autres colonnes
		for(int32 index=0;index<_itemListRect.CountItems();index++)
		{
			columnRect = (BRect *)(_itemListRect.ItemAt(index));
			if(columnRect!=resizeRect && columnRect->right>resizeRect->right)
				columnRect->OffsetBy(offset,0);
		}
		
		// reafficher
		offset = abs(offset);
		Draw(BRect(resizeRect->right,1,bounds.right,bounds.bottom-1));
		_listView->Invalidate();
	}
	
	// ok tout c'est bien passé
	return B_OK;
}

/**** trouver si on se trouve dans la zone de redimentionnement d'une colonne ****/
int32 CColumnView::InColumnResizeArea(BPoint where)
{
	int32	count;
	BRect	columnRect;
	
	count = _itemListRect.CountItems();
	for(int32 index=0; index<count; index++)
	{
		columnRect = *((BRect *)(_itemListRect.ItemAt(index)));
		columnRect.left = columnRect.right - EDGE_RESIZE;
		columnRect.right += EDGE_RESIZE;
		if(columnRect.Contains(where))
			return index;
	}

	return -1;
}


/**** ajouter une colonne ****/
bool CColumnView::AddItem(CColumnItem *newItem,float neededSize)
{
	if(newItem==NULL)
		return false;
		
	BRect		bounds = Bounds();
	bool		state = true;	
	float		startPosition = SIZE_START_COLOMN;
	float		labelSize = StringWidth(newItem->Label()) + SIZE_LABEL_MORE;
	int32		nbItems = _itemListRect.CountItems();

	// on peux demander une taille precise
	if(neededSize>labelSize)
		labelSize = neededSize;
		
	// calculer la position de depart
	for(int32 index=0;index<nbItems;index++)
		startPosition += ((BRect *)(_itemListRect.ItemAt(index)))->Width() + 1;

	// ajouter l'item et l'info sur ce dernier
	state &= _itemList.AddItem(newItem);
	state &= _itemListRect.AddItem(new BRect(startPosition,1,startPosition + labelSize,bounds.Height() - 1));

	return state;
}
