/***********************************************/
/* Software Created By CKJ Vincent Cedric      */
/* This code is the property of Vincent Cedric */
/* You can't copy or use this code without     */
/* according of the author (GPL License).	   */
/***********************************************/
/*****************************************/
/* MeTOS Language class for localisation */
/*****************************************/
#include "VLanguage.h"
#include "MetosGlobalDefs.h"
#include "MetosMessageDefs.h"
#include "MetosLanguagesDefs.h"
#include "VPreferenceFile.h"

#include <storage/File.h>
#include <storage/Path.h>
#include <app/Roster.h>
#include <app/Application.h>

#include <stdlib.h>

/*
 * yT: posix locale
 *
 * Taken from Francois. Modified by Michael.
 * We use English for languages which are not available.
 * As soon as other languages are added, we need to add them here.
 * (Note: Namings according to filenames.)
 */

static const struct {
	const char* posix;
	const char* language;
} posixLocaleMapping[] = {
	{ "C", "English" }, // the default one
	{ "en", "English" },
	{ "es", "English" },
	{ "de", "German" },
	{ "nl", "English" },
	{ "pt", "English" },
	{ "fr", "Français" },
	{ "it", "English" },
	{ "ru", "English" },
	{ "sv", "English" },
	{ "eo", "English" },
	{ "no", "English" },
	{ "sr", "English" },
	{ "bg", "English" },
	{ "ko", "English" },
	{ "hu", "English" },
	{ "ro", "English" },
	{ "zh", "English" },  // simplified Chinese
	{ "tk", "English" },
	{ NULL, 0 }
};

/**** constructeur ****/
VLanguage::VLanguage(const char *filename)
{
//	printf("VLanguage::VLanguage(%s)\n", filename);
	// par defaut c'est pas bon
	_state = B_ERROR;

	// verifier le nom de fichier
	if(filename==NULL)
		return;

	// creer le message porteur des traductions
	_translations = new BMessage(M_LANGUAGE_MSG);
	_filename = filename;
	
	// ok
	_state = B_OK;
}

/**** destructeur ****/
VLanguage::~VLanguage()
{
	// liberer la memoire
	if(_translations!=NULL)
		delete _translations;
}

/**** charger les données du fichiers ****/
status_t VLanguage::Load()
{
//	printf("VLanguage::Load()\n");
	// verifier l'etat de construction
	if(InitCheck()!=B_OK)
		return B_ERROR;

	// on doit avoir un pointeur d'application
	if(be_app==NULL)
		return B_ERROR;		

	BFile		file;
	BEntry		entry;
	app_info	info;
	BPath		appPath;
	BPath		languagePath;
	BString		languageFile;

	// initialiser le fichier ressource
	// et le chemin de l'appli
	be_app->GetAppInfo(&info);
	file.SetTo(&(info.ref), B_READ_ONLY);
	appPath.SetTo(&(info.ref)); 
    appPath.GetParent(&appPath);
	
	// on va construire le chemin
	languageFile.SetTo(appPath.Path());
	languageFile << "/";
	languageFile << ADDON_FOLDER;
	languageFile << "/";
	languageFile << METOS_LANGUAGES_FOLDER;
	languageFile << "/";
	languageFile << _filename;
	languageFile << ".";
	
	/* old */
	//languageFile << g_MeTOSPrefs->GetString("English",METOS_LANGUAGES_TYPE);
	
	/* yT: posix locale switch */
	
	BString posixLocale( getenv( "LC_ALL" ) );
	BString metosLocale( "English" );	// set to default
	if( !posixLocale.Length() )
		posixLocale = getenv( "LC_MESSAGES" );
	
	if( posixLocale.Length() )
	{
		if( posixLocale.FindFirst( '_' ) >= 0 )
			posixLocale.Truncate( posixLocale.FindFirst( '_' ) );
		
		for( int i = 0; posixLocaleMapping[i].posix; i++ )
		{
			if( strcmp( posixLocale.String(), posixLocaleMapping[i].posix ) == 0 )
			{
//				printf( "  strings match: posix: %s metos: %s\n",
//					posixLocaleMapping[i].posix,
//					posixLocaleMapping[i].language );
				
				metosLocale.SetTo( posixLocaleMapping[i].language );
				
				break;
			}
		}
	}
	/* append correct language */
	languageFile << metosLocale.String();
	
//	printf( "  langueageFile: %s\n", languageFile.String() );

	/* end of yT: posix locale switch */	

	languagePath.SetTo(languageFile.String());
	if(languagePath.InitCheck()!=B_OK)
		return B_ERROR;
	
	// puis lire du fichier texte les IDs et la traduction qui va avec
	// si la trduction n'est pas la on gardera LANGUAGE_NO_TRANSLATION
	BFile		textFile(languageFile.String(),B_READ_ONLY);
	char		letter;
	char		id[255];
	char		word[4096];
	char		*buffer = NULL;
	short		index;
	
	// verifier la validité du fichier
	if(textFile.InitCheck()!=B_OK)
		return B_ERROR;

	// on va en premier recuperer un ID
	index = 0;
	buffer = id;

	// charger la traduction
	textFile.Seek(0,SEEK_SET);
	while(textFile.Read(&letter,1)==1)
	{
		// voir en fonction de la lettre que l'on recupere
		switch(letter)
		{
		// fin de ligne et donc de traduction
		// on ajoute l'ID et la traduction
		case LANGUAGE_END_OF_LINE:
			buffer[index] = '\0';
			_translations->AddString(id,word);
			id[0] = '\0';
			word[0] = '\0';
			index = 0;
			buffer = id;
			break;
		// separateur, fin de l'ID
		case LANGUAGE_SEPARATOR:
			buffer[index] = '\0';
			index = 0;
			buffer = word;
			break;
		default:
			buffer[index] = letter;
			index++;
		}
	}
		
	// ok tout c'est bien passé
	return B_OK;
}

/**** recuperer la traduction ****/
const char *VLanguage::GetTranslationFor(const char *id)
{
//	printf("looking up id: %s, this: %p\n", id, this);
	const char	*translation = NULL;

//	printf("this: %p, _translations: %p\n", this, _translations);
//	_translations->PrintToStream();
//	printf("------------------------\n");	
	// on va essayer de trouver la correspondance dans le message
	status_t result = B_OK;
	if(_translations!=NULL)
		result = _translations->FindString(id,&translation);
	else
		translation = LANGUAGE_NO_TRANSLATION;
//	printf("result: %ld, %s\n", result, strerror(result));
	
	// on a rien trouvé
	return translation;
}

/**** verifier l'etat d'initialisation ****/
status_t VLanguage::InitCheck()
{	return _state;	}

